unit xidcusb;

interface

uses Windows, Sysutils;

const
	XidcusbDll = 'libxidcusb.dll';

type
	Device = integer;

type
	XidcResult = integer;

type
        StringArray = array of string;

const
	DeviceUndefined = -1;

const
	ResultOk = 0;
	ResultError = -1;
	ResultNotImplemented = -2;
	ResultDataError = -3;
	ResultNoDevice = -4;


// Describes device state
const
	DevStateOff = 0;
	DevStateStop = 1;
	DevStateMoving = 2;
	DevStateTune = 3;
	DevStateCalibr = 0x0C;
	DevStateAlarm = 0x0A;

// Describes PWM state
const
	PwmStateOff = 0;
	PwmStateBrak = 1;
	PwmStateRunfwd = 2;
	PwmStateRunbck = 3;
	PwmStateInvbrak	= 4;

// Describes feedback type
const
	FeedbackPotentiometer = 0;
	FeedbackEncoder = 1;
	FeedbackEncoderDiff = 2;
	FeedbackEncoderHall = 4;
	FeedbackEncoderEMF = 8;

// Describes synchronization pin mode
const
	PinGPIO			= 0x00;
	PinSyncIn		= 0x01;
	PinSyncOut	= 0x02;

// Status flags
const
	 STATE_RIGHT_EDGE        = 0x0001;
	 STATE_LEFT_EDGE         = 0x0002;
	 STATE_EXT_POWER         = 0x0004;
	 STATE_ERRC              = 0x0008;
	 STATE_ERRD              = 0x0010;
	 STATE_POWER_DISABLE     = 0x0020;
	 STATE_TTLIO_PINOUT      = 0x0040;
	 STATE_TTLIO_LEVEL       = 0x0080;
	 STATE_BUTTON_RIGHT      = 0x0100;
	 STATE_BUTTON_LEFT       = 0x0200;
	 STATE_OVERHEAT          = 0x2000;
	 STATE_OVERLOAD_VOLTAGE  = 0x4000;
	 STATE_OVERLOAD_CURRENT  = 0x8000;

// Flags of engine settings
const
	 ENGINE_REVERSE        	= 0x0001;
	 ENGINE_FINISHING        = 0x0002;
	 ENGINE_MAX_SPEED        = 0x0004;
	 ENGINE_ANTIPLAY         = 0x0008;
	 ENGINE_ACCEL_ON         = 0x0010;
	 ENGINE_LIMIT_VOLT      	= 0x0020;
	 ENGINE_LIMIT_CURR     	= 0x0040;
	 ENGINE_LIMIT_RPM       	= 0x0080;
	 ENGINE_HOLD  			= 0x0100;
	 ENGINE_DYNAMIC_HOLD  	= 0x0200;
	 ENGINE_ONLY_FINISHING   = 0x0400;

// Flags of synchronization setup
const
	 TTL_SETUP_DIROUT	= 0x0001;
	 TTL_SETUP_OUTSET	= 0x0002;
	 TTL_SETUP_FRONT		= 0x0010;

// Flags of synchronization input
const
	 TTL_SYNCIN_ONMOVE				= 0x0001;
	 TTL_SYNCIN_MULTI_SHIFT	= 0x0004;
	 TTL_SYNCIN_DIRRIGHT			= 0x0008;

// Flags of synchronization output
const
	 TTL_SYNCOUT_ONSTART		= 0x0001;
	 TTL_SYNCOUT_ONSTOP		= 0x0002;
	 TTL_SYNCOUT_ONPERIOD	= 0x0004;

// Border flags
const
	 BORDER_IS_ENCODER	= 0x0001;
	 BORDER_STOP_LEFT	= 0x0002;
	 BORDER_STOP_RIGHT	= 0x0004;

// Ender flags
const
	 ENDER_DIRECT	= 0x0001;
	 ENDER_ON1			= 0x0002;
	 ENDER_ON2			= 0x0004;

// Home flags
const
	 HOME_TODO1 = 0x0001;
	 HOME_TODO2 = 0x0002;
	 HOME_TODO3 = 0x0004;
	 HOME_TODO4 = 0x0008;

// Device state
type State = record

	device_state : integer;
	pwm_state : integer;
	flags : cardinal;
	current_position : cardinal;
	current_rpm : integer;
	current_input : integer;
	current_engine_voltage : integer;
	current_power_voltage : integer;
	current_temp : integer;
end;

// Calibration settings
type CalibrationSettings = record

	amp_vout1 : integer;
	amp_vout2 : integer;
	amp_vext : integer;
	amp_vkey : integer;
	amp_engcur : integer;
	amp_tempkey : integer;
	off_vout1 : integer;
	off_vout2 : integer;
	off_vext : integer;
	off_vkey : integer;
	off_engcur : integer;
	off_tempkey : integer;
end;

// Engine settings
type EngineSettings = record

	nom_voltage : cardinal;
	nom_input : cardinal;
	nom_rpm : cardinal;
	flags : cardinal;
	antiplay : integer;
end;

// PID settings
type PidSettings = record

	kpi : cardinal;
	kii : cardinal;
	kdi : cardinal;
	kpu : cardinal;
	kiu : cardinal;
	kdu : cardinal;
	kprpm : cardinal;
	kirpm : cardinal;
	kdrpm : cardinal;
	kppos : cardinal;
	kipos : cardinal;
	kdpos : cardinal;
	kpe1 : cardinal;
	kie1 : cardinal;
	kde1 : cardinal;
	kpe2 : cardinal;
	kie2 : cardinal;
	kde2 : cardinal;
	kpe3 : cardinal;
	kie3 : cardinal;
	kde3 : cardinal;
end;

// Synchronization settings
type SyncSettings = record

	setup_flags : cardinal;
	syncin_flags : cardinal;
	syncout_flags : cardinal;
	sync_pin_mode : integer;
	impulse_time : cardinal;
	impulse_period : cardinal;
end;

// Home settings
type HomeSettings = record

	fast_home : integer;
	slow_home : integer;
	delta : cardinal;
	flags : integer;
end;


// Analog settings
type AnalogData = record

		adc_va1 : integer;			
		adc_va2 : integer;			
		adc_vb1 : integer;				
		adc_vext : integer;				
		adc_vsup : integer;				
		adc_acur : integer;			
		adc_bcur : integer;			
		adc_fullcur : integer;			
		adc_pot : integer;				
		adc_tempkey : integer;		
		cdc_va1 : integer;			
		cdc_va2 : integer;			
		cdc_vb1 : integer;				
		cdc_vext : integer;				
		cdc_vsup : integer;				
		cdc_acur : integer;				
		cdc_bcur : integer;				
		cdc_fullcur : integer;				
		cdc_tempkey : integer;		
		cdc_dutycycle : integer;			
end;

// Chart data
type ChartData = record
	winding_voltage_a : integer;
	winding_voltage_b : integer;
	winding_voltage_c : integer;
	winding_current_a : integer;
	winding_current_b : integer;
	winding_current_c : integer;
	current_temp : integer;
end;

function open_device (name: PAnsiChar) : Device; stdcall; external XidcusbDll;
function open_raw_device (name: PAnsiChar) : Device; stdcall; external XidcusbDll;
function close_device (id: Device) : XidcResult; stdcall; external XidcusbDll;
function probe_device (name: PAnsiChar) : XidcResult; stdcall; external XidcusbDll;

function enumerate_devices(probe_devices: Integer): StringArray;

function init_device (id: Device) : XidcResult; stdcall; external XidcusbDll;
function deinit_device (id: Device) : XidcResult; stdcall; external XidcusbDll;
function reset_locks (id: Device) : XidcResult; stdcall; external XidcusbDll;
function get_device_information (id: Device; out manufacturer, manufacturer_id, product_description: PAnsiChar) : XidcResult; stdcall; external XidcusbDll;
function get_status (id: Device; out state: State) : XidcResult; stdcall; external XidcusbDll;

function command_read_settings(id: Device) : XidcResult; stdcall; external XidcusbDll;
function command_save_settings(id: Device) : XidcResult; stdcall; external XidcusbDll;
function get_serial_number(id: Device; out serial: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function set_serial_number(id: Device; serial, key: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function get_firmware_version(id: Device; out major, minor, release: Integer) : XidcResult; stdcall; external XidcusbDll;
function has_firmware(id: Device; out ret: Byte) : XidcResult; stdcall; external XidcusbDll;
function write_key(id: Device; key: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function command_calibrate(id: Device) : XidcResult; stdcall; external XidcusbDll;
function command_reset(id: Device) : XidcResult; stdcall; external XidcusbDll;
function command_update_firmware(id: Device; data: PByte; data_size: Cardinal) : XidcResult; stdcall; external XidcusbDll;

function command_stop (id: Device) : XidcResult; stdcall; external XidcusbDll;
function command_zero (id: Device) : XidcResult; stdcall; external XidcusbDll;
function command_move (id: Device; pos: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function command_movr (id: Device; offset: Integer) : XidcResult; stdcall; external XidcusbDll;
function command_left (id: Device) : XidcResult; stdcall; external XidcusbDll;
function command_right (id: Device) : XidcResult; stdcall; external XidcusbDll;

function get_calibration_coeffs(id: Device; out cs: CalibrationSettings) : XidcResult; stdcall; external XidcusbDll;
function set_calibration_coeffs(id: Device; var cs: CalibrationSettings) : XidcResult; stdcall; external XidcusbDll;
function get_pwm_freq (id: Device; out freq: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function set_pwm_freq (id: Device; freq: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function get_feedback_settings (id: Device; out ips: Cardinal; out feedback_flags: Integer) : XidcResult; stdcall; external XidcusbDll;
function set_feedback_settings (id: Device; ips: Cardinal; feedback_flags: Integer) : XidcResult; stdcall; external XidcusbDll;
function get_move_settings (id: Device; out rpm, accel, tuneup_threshold: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function set_move_settings (id: Device; rpm, accel, tuneup_threshold: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function get_engine_settings (id: Device; out engine_settings: EngineSettings) : XidcResult; stdcall; external XidcusbDll;
function set_engine_settings (id: Device; var engine_settings: EngineSettings) : XidcResult; stdcall; external XidcusbDll;
function get_secure_settings (id: Device; out critical_curr, critical_voltage, critical_temp: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function set_secure_settings (id: Device; critical_curr, critical_voltage, critical_temp: Cardinal) : XidcResult; stdcall; external XidcusbDll;
function get_edges_settings (id: Device; out border_flags, enders_flags, left, right: Integer) : XidcResult; stdcall; external XidcusbDll;
function set_edges_settings (id: Device; border_flags, enders_flags, left, right: Integer) : XidcResult; stdcall; external XidcusbDll;
function get_pid_settings (id: Device; out pid_settings: PidSettings) : XidcResult; stdcall; external XidcusbDll;
function set_pid_settings (id: Device; var pid_settings: PidSettings) : XidcResult; stdcall; external XidcusbDll;
function get_sync_settings (id: Device; out sync_settings: SyncSettings) : XidcResult; stdcall; external XidcusbDll;
function set_sync_settings (id: Device; var sync_settings: SyncSettings) : XidcResult; stdcall; external XidcusbDll;
function get_home_settings (id: Device; out home_settings: HomeSettings) : XidcResult; stdcall; external XidcusbDll;
function set_home_settings (id: Device; var home_settings: HomeSettings) : XidcResult; stdcall; external XidcusbDll;
function get_analog_data(id: Device; out ad: AnalogData) : XidcResult; stdcall; external XidcusbDll;
function get_chart_data(id: Device; out cd: ChartData) : XidcResult; stdcall; external XidcusbDll;

implementation

type
        PPAnsiChar = ^PAnsiChar;
function enumerate_devices_impl(var name_count: Integer; out names: PPAnsiChar; probe_devices: Integer) : XidcResult;
        stdcall; external XidcusbDll name 'enumerate_devices';
function free_enumerate_devices_impl (name_count: Integer; names: PPAnsiChar) : XidcResult;
        stdcall; external XidcusbDll name 'free_enumerate_devices';

function enumerate_devices(probe_devices: Integer): StringArray;
type
        FixedArray = array [0..65535] of PAnsiChar;
        PFixedArray = ^FixedArray;
var
        res: XidcResult;
        name_count: Integer;
        names_ptr: PPAnsiChar;
        i: Integer;
        pre_buf_ptr: PFixedArray;
begin
        res := enumerate_devices_impl(name_count, names_ptr, probe_devices);
        if (res = ResultOk) and (name_count > 0) then begin
                pre_buf_ptr := PFixedArray(names_ptr);
                SetLength(Result, name_count);
                for i := 0 to name_count-1 do
                        Result[i] := pre_buf_ptr^[i];
        end;
        free_enumerate_devices_impl(name_count, names_ptr);
end;

end.
